/// <reference types="node" />
import { PublicKey } from '@solana/web3.js';
/**
 * A port of Metaplex's Metadata layout.
 *
 * Metaplex library introduces extra dependencies that requires node
 * polyfill, And we only use a small set of its features.
 */
export interface IMetadata {
    key: number;
    updateAuthority: PublicKey;
    mint: PublicKey;
    data: Data;
    primarySaleHappened: boolean;
    isMutable: boolean;
    editionNonce: number | undefined;
    tokenStandard: TokenStandard | undefined;
    collection: ICollection | undefined;
    uses: IUses | undefined;
}
export interface ICollection {
    verified: boolean;
    key: PublicKey;
}
export declare const USE_METHOD: {
    readonly Burn: 0;
    readonly Multiple: 1;
    readonly Single: 2;
};
type UseMethod = typeof USE_METHOD[keyof typeof USE_METHOD];
export interface IUses {
    useMethod: UseMethod;
    remaining: bigint;
    total: bigint;
}
export declare const TOKEN_STANDARD: {
    readonly NonFungible: 0;
    readonly FungibleAsset: 1;
    readonly Fungible: 2;
    readonly NonFungibleEdition: 3;
    readonly ProgrammableNonFungible: 4;
};
export type TokenStandard = typeof TOKEN_STANDARD[keyof typeof TOKEN_STANDARD];
export interface ICreator {
    address: PublicKey;
    verified: boolean;
    share: number;
}
export interface IData {
    name: string;
    symbol: string;
    uri: string;
    sellerFeeBasisPoints: number;
    creators: ICreator[] | undefined;
}
export declare class Creator implements ICreator {
    address: PublicKey;
    verified: boolean;
    share: number;
    constructor(fields: ICreator);
}
export declare class Data implements IData {
    name: string;
    symbol: string;
    uri: string;
    sellerFeeBasisPoints: number;
    creators: ICreator[] | undefined;
    constructor(fields: IData);
}
export declare class Collection implements ICollection {
    verified: boolean;
    key: PublicKey;
    constructor(fields: IData);
}
export declare class Uses implements IUses {
    useMethod: UseMethod;
    remaining: bigint;
    total: bigint;
    constructor(fields: IData);
}
export declare class Metadata implements IMetadata {
    key: number;
    updateAuthority: PublicKey;
    mint: PublicKey;
    data: Data;
    primarySaleHappened: boolean;
    isMutable: boolean;
    editionNonce: number | undefined;
    tokenStandard: TokenStandard | undefined;
    collection: Collection | undefined;
    uses: Uses | undefined;
    constructor(fields: IMetadata);
    static deserialize(data: Buffer): Metadata;
}
export {};
