import { PublicKey } from '@solana/web3.js';
import { IWallet, ITransport, CreateGameAccountParams, CloseGameAccountParams, JoinParams, DepositParams, VoteParams, CreatePlayerProfileParams, PublishGameParams, CreateRegistrationParams, RegisterGameParams, UnregisterGameParams, GameAccount, GameBundle, PlayerProfile, ServerAccount, RegistrationAccount, IToken, INft, RegistrationWithGames } from '@race-foundation/sdk-core';
import { GameState, RegistryState, ServerState } from './accounts';
export declare class SolanaTransport implements ITransport {
    #private;
    constructor(endpoint: string);
    _fetchLegacyTokens(): Promise<void>;
    createGameAccount(wallet: IWallet, params: CreateGameAccountParams): Promise<string>;
    closeGameAccount(wallet: IWallet, params: CloseGameAccountParams): Promise<void>;
    join(wallet: IWallet, params: JoinParams): Promise<void>;
    deposit(wallet: IWallet, params: DepositParams): Promise<void>;
    publishGame(wallet: IWallet, params: PublishGameParams): Promise<string>;
    vote(wallet: IWallet, params: VoteParams): Promise<void>;
    createPlayerProfile(wallet: IWallet, params: CreatePlayerProfileParams): Promise<void>;
    createRegistration(wallet: IWallet, params: CreateRegistrationParams): Promise<string>;
    registerGame(wallet: IWallet, params: RegisterGameParams): Promise<void>;
    unregisterGame(wallet: IWallet, params: UnregisterGameParams): Promise<void>;
    getGameAccount(addr: string): Promise<GameAccount | undefined>;
    getGameBundle(addr: string): Promise<GameBundle | undefined>;
    getPlayerProfile(addr: string): Promise<PlayerProfile | undefined>;
    getServerAccount(addr: string): Promise<ServerAccount | undefined>;
    getRegistration(addr: string): Promise<RegistrationAccount | undefined>;
    getRegistrationWithGames(addr: string): Promise<RegistrationWithGames | undefined>;
    _fetchImageFromDataUri(dataUri: string): Promise<string | undefined>;
    getToken(addr: string): Promise<IToken | undefined>;
    /**
     * List popular tokens.
     *
     * [USDT, USDC, SOL, RACE]
     */
    listTokens(): Promise<IToken[]>;
    fetchBalances(walletAddr: string, tokenAddrs: string[]): Promise<Map<string, bigint>>;
    getNft(addr: string | PublicKey): Promise<INft | undefined>;
    listNfts(walletAddr: string): Promise<INft[]>;
    _getGameState(gameAccoutKey: PublicKey): Promise<GameState | undefined>;
    _getMultiGameStates(gameAccountKeys: PublicKey[]): Promise<Array<GameState | undefined>>;
    _getRegState(regKey: PublicKey): Promise<RegistryState | undefined>;
    _getServerState(serverKey: PublicKey): Promise<ServerState | undefined>;
}
