/// <reference types="node" />
import { PublicKey, TransactionInstruction } from '@solana/web3.js';
export declare enum Instruction {
    CreateGameAccount = 0,
    CloseGameAccount = 1,
    CreateRegistration = 2,
    CreatePlayerProfile = 3,
    RegisterServer = 4,
    Settle = 5,
    Vote = 6,
    ServeGame = 7,
    RegisterGame = 8,
    UnregisterGame = 9,
    JoinGame = 10,
    PublishGame = 11
}
declare abstract class Serialize {
    serialize(): Buffer;
}
export declare class CreatePlayerProfileData extends Serialize {
    instruction: Instruction;
    nick: string;
    constructor(nick: string);
}
export declare class CloseGameAccountData extends Serialize {
    instruction: Instruction;
    constructor();
}
export declare class CreateGameAccountData extends Serialize {
    instruction: Instruction;
    title: string;
    maxPlayers: number;
    minDeposit: bigint;
    maxDeposit: bigint;
    data: Uint8Array;
    constructor(params: Partial<CreateGameAccountData>);
}
export declare class JoinGameData extends Serialize {
    instruction: Instruction;
    amount: bigint;
    accessVersion: bigint;
    position: number;
    verifyKey: string;
    constructor(amount: bigint, accessVersion: bigint, position: number, verifyKey: string);
}
export declare class PublishGameData extends Serialize {
    instruction: Instruction;
    uri: string;
    name: string;
    symbol: string;
    constructor(uri: string, name: string, symbol: string);
}
export declare function createPlayerProfile(ownerKey: PublicKey, profileKey: PublicKey, nick: string, pfpKey?: PublicKey): TransactionInstruction;
export type CreateGameOptions = {
    ownerKey: PublicKey;
    gameAccountKey: PublicKey;
    stakeAccountKey: PublicKey;
    mint: PublicKey;
    gameBundleKey: PublicKey;
    title: string;
    maxPlayers: number;
    minDeposit: bigint;
    maxDeposit: bigint;
};
export declare function createGameAccount(opts: CreateGameOptions): TransactionInstruction;
export type CloseGameAccountOptions = {
    ownerKey: PublicKey;
    gameAccountKey: PublicKey;
    regAccountKey: PublicKey;
    gameStakeKey: PublicKey;
};
export declare function closeGameAccount(opts: CloseGameAccountOptions): TransactionInstruction;
export type JoinOptions = {
    playerKey: PublicKey;
    paymentKey: PublicKey;
    gameAccountKey: PublicKey;
    mint: PublicKey;
    stakeAccountKey: PublicKey;
    amount: bigint;
    accessVersion: bigint;
    position: number;
    verifyKey: string;
};
export declare function join(opts: JoinOptions): Promise<TransactionInstruction>;
export type PublishGameOptions = {
    ownerKey: PublicKey;
    mint: PublicKey;
    tokenAccountKey: PublicKey;
    uri: string;
    name: string;
    symbol: string;
};
export declare function publishGame(opts: PublishGameOptions): TransactionInstruction;
export {};
//# sourceMappingURL=instruction.d.ts.map